@@ -48,7 +48,11 @@ pub enum Instruction {
         indices: Vec<u64>,
     },
     /// Return a pointer as a value.
-    GetPointer(Pointer),
+    GetPointer {
+        base_ptr: Pointer,
+        ptr_ty: Type,
+        offset: u64,
+    },
     /// Writing a specific value to an array.
     InsertElement {
         array: Value,
@@ -69,16 +73,19 @@ pub enum Instruction {
     Nop,
     /// Choose a value from a list depending on the preceding block.
     Phi(Vec<(Block, Value)>),
-    /// A cast from one pointer type to another.  Value must be either a GetPointer instruction or
-    /// another PointerCast.
-    PointerCast(Value, Type),
     /// Return from a function.
     Ret(Value, Type),
-    /// Read a value from a storage slot.  Type of `load_val` must be a Uint(64) or B256 ptr.
-    StateLoad { load_val: Value, key: Value },
+    /// Read a quad word from a storage slot. Type of `load_val` must be a B256 ptr.
+    StateLoadQuadWord { load_val: Value, key: Value },
+    /// Read a single word from a storage slot.
+    StateLoadWord(Value),
     /// Write a value to a storage slot.  Key must be a B256, type of `stored_val` must be a
-    /// Uint(64) or B256 ptr.
-    StateStore { stored_val: Value, key: Value },
+    /// Uint(256) ptr.
+    StateStoreQuadWord { stored_val: Value, key: Value },
+    /// Write a value to a storage slot.  Key must be a B256, type of `stored_val` must be a
+    /// Uint(64) value.
+    StateStoreWord { stored_val: Value, key: Value },
+
     /// Write a value to a memory pointer.
     Store { dst_val: Value, stored_val: Value },
 }
@@ -106,8 +113,7 @@ impl Instruction {
             }
 
             // These can be recursed to via Load, so we return the pointer type.
-            Instruction::GetPointer(ptr) => Some(context.pointers[ptr.0].ty),
-            Instruction::PointerCast(_, ty) => Some(*ty),
+            Instruction::GetPointer { ptr_ty, .. } => Some(*ptr_ty),
 
             // These are all terminators which don't return, essentially.  No type.
             Instruction::Branch(_) => None,
@@ -117,8 +123,10 @@ impl Instruction {
             // These write values but don't return one.  If we're explicit we could return Unit.
             Instruction::InsertElement { .. } => None,
             Instruction::InsertValue { .. } => None,
-            Instruction::StateLoad { .. } => None,
-            Instruction::StateStore { .. } => None,
+            Instruction::StateLoadQuadWord { .. } => None,
+            Instruction::StateLoadWord(_) => Some(Type::Uint(64)),
+            Instruction::StateStoreQuadWord { .. } => None,
+            Instruction::StateStoreWord { .. } => None,
             Instruction::Store { .. } => None,
 
             // No-op is also no-type.
@@ -129,7 +137,7 @@ impl Instruction {
     /// Some [`Instruction`]s may have struct arguments.  Return it if so for this instruction.
     pub fn get_aggregate(&self, context: &Context) -> Option<Aggregate> {
         match self {
-            Instruction::GetPointer(ptr) => match ptr.get_type(context) {
+            Instruction::GetPointer { ptr_ty, .. } => match ptr_ty {
                 Type::Array(aggregate) => Some(*aggregate),
                 Type::Struct(aggregate) => Some(*aggregate),
                 _otherwise => None,
@@ -172,7 +180,7 @@ impl Instruction {
             Instruction::Branch(_) => (),
             Instruction::Call(_, args) => args.iter_mut().for_each(replace),
             Instruction::ConditionalBranch { cond_value, .. } => replace(cond_value),
-            Instruction::GetPointer(_) => (),
+            Instruction::GetPointer { .. } => (),
             Instruction::InsertElement {
                 array,
                 value,
@@ -199,13 +207,19 @@ impl Instruction {
             Instruction::Load(_) => (),
             Instruction::Nop => (),
             Instruction::Phi(pairs) => pairs.iter_mut().for_each(|(_, val)| replace(val)),
-            Instruction::PointerCast(..) => (),
             Instruction::Ret(ret_val, _) => replace(ret_val),
-            Instruction::StateLoad { load_val, key } => {
+            Instruction::StateLoadQuadWord { load_val, key } => {
                 replace(load_val);
                 replace(key);
             }
-            Instruction::StateStore { stored_val, key } => {
+            Instruction::StateLoadWord(key) => {
+                replace(key);
+            }
+            Instruction::StateStoreQuadWord { stored_val, key } => {
+                replace(key);
+                replace(stored_val);
+            }
+            Instruction::StateStoreWord { stored_val, key } => {
                 replace(key);
                 replace(stored_val);
             }
@@ -399,9 +413,22 @@ impl<'a> InstructionInserter<'a> {
         extract_value_val
     }
 
-    pub fn get_ptr(self, ptr: Pointer, span_md_idx: Option<MetadataIndex>) -> Value {
-        let get_ptr_val =
-            Value::new_instruction(self.context, Instruction::GetPointer(ptr), span_md_idx);
+    pub fn get_ptr(
+        self,
+        base_ptr: Pointer,
+        ptr_ty: Type,
+        offset: u64,
+        span_md_idx: Option<MetadataIndex>,
+    ) -> Value {
+        let get_ptr_val = Value::new_instruction(
+            self.context,
+            Instruction::GetPointer {
+                base_ptr,
+                ptr_ty,
+                offset,
+            },
+            span_md_idx,
+        );
         self.context.blocks[self.block.0]
             .instructions
             .push(get_ptr_val);
@@ -471,18 +498,6 @@ impl<'a> InstructionInserter<'a> {
         nop_val
     }
 
-    pub fn ptr_cast(self, ptr_val: Value, ty: Type, span_md_idx: Option<MetadataIndex>) -> Value {
-        let ptr_cast_val = Value::new_instruction(
-            self.context,
-            Instruction::PointerCast(ptr_val, ty),
-            span_md_idx,
-        );
-        self.context.blocks[self.block.0]
-            .instructions
-            .push(ptr_cast_val);
-        ptr_cast_val
-    }
-
     pub fn ret(self, value: Value, ty: Type, span_md_idx: Option<MetadataIndex>) -> Value {
         let ret_val =
             Value::new_instruction(self.context, Instruction::Ret(value, ty), span_md_idx);
@@ -490,7 +505,7 @@ impl<'a> InstructionInserter<'a> {
         ret_val
     }
 
-    pub fn state_load(
+    pub fn state_load_quad_word(
         self,
         load_val: Value,
         key: Value,
@@ -498,7 +513,7 @@ impl<'a> InstructionInserter<'a> {
     ) -> Value {
         let state_load_val = Value::new_instruction(
             self.context,
-            Instruction::StateLoad { load_val, key },
+            Instruction::StateLoadQuadWord { load_val, key },
             span_md_idx,
         );
         self.context.blocks[self.block.0]
@@ -507,7 +522,16 @@ impl<'a> InstructionInserter<'a> {
         state_load_val
     }
 
-    pub fn state_store(
+    pub fn state_load_word(self, key: Value, span_md_idx: Option<MetadataIndex>) -> Value {
+        let state_load_val =
+            Value::new_instruction(self.context, Instruction::StateLoadWord(key), span_md_idx);
+        self.context.blocks[self.block.0]
+            .instructions
+            .push(state_load_val);
+        state_load_val
+    }
+
+    pub fn state_store_quad_word(
         self,
         stored_val: Value,
         key: Value,
@@ -515,7 +539,24 @@ impl<'a> InstructionInserter<'a> {
     ) -> Value {
         let state_store_val = Value::new_instruction(
             self.context,
-            Instruction::StateStore { stored_val, key },
+            Instruction::StateStoreQuadWord { stored_val, key },
+            span_md_idx,
+        );
+        self.context.blocks[self.block.0]
+            .instructions
+            .push(state_store_val);
+        state_store_val
+    }
+
+    pub fn state_store_word(
+        self,
+        stored_val: Value,
+        key: Value,
+        span_md_idx: Option<MetadataIndex>,
+    ) -> Value {
+        let state_store_val = Value::new_instruction(
+            self.context,
+            Instruction::StateStoreWord { stored_val, key },
             span_md_idx,
         );
         self.context.blocks[self.block.0]
