@@ -31,6 +31,9 @@ mod ir_builder {
                 = _ s:script() eoi() {
                     s
                 }
+                / _ s:contract() eoi() {
+                    s
+                }
 
             rule script() -> IrAstModule
                 = "script" _ "{" _ fn_decls:fn_decl()* "}" _ metadata:metadata_decl()* {
@@ -41,8 +44,18 @@ mod ir_builder {
                     }
                 }
 
+            rule contract() -> IrAstModule
+                = "contract" _ "{" _ fn_decls:fn_decl()* "}" _ metadata:metadata_decl()* {
+                    IrAstModule {
+                        kind: crate::module::Kind::Contract,
+                        fn_decls,
+                        metadata
+                    }
+                }
+
             rule fn_decl() -> IrAstFnDecl
-                = "fn" _ name:id() "(" _ args:(fn_arg() ** comma()) ")" _ "->" _ ret_type:ast_ty() "{" _
+                = "fn" _ name:id() _ selector:selector_id()? _ "(" _
+                      args:(fn_arg() ** comma()) ")" _ "->" _ ret_type:ast_ty() "{" _
                       locals:fn_local()*
                       blocks:block_decl()*
                   "}" _ {
@@ -52,9 +65,17 @@ mod ir_builder {
                         ret_type,
                         locals,
                         blocks,
+                        selector
                     }
                 }
 
+
+
+            rule selector_id() -> [u8; 4]
+                = "<" _ s:$(['0'..='9' | 'a'..='f' | 'A'..='F']*<8>) _ ">" _ {
+                    string_to_hex::<4>(s)
+                }
+
             rule fn_arg() -> (IrAstTy, String, Option<MdIdxRef>)
                 = name:id() mdi:metadata_idx()? ":" _ ty:ast_ty() {
                     (ty, name, mdi)
@@ -116,10 +137,11 @@ mod ir_builder {
                 / op_load()
                 / op_nop()
                 / op_phi()
-                / op_ptr_cast()
                 / op_ret()
-                / op_state_load()
-                / op_state_store()
+                / op_state_load_quad_word()
+                / op_state_load_word()
+                / op_state_store_quad_word()
+                / op_state_store_word()
                 / op_store()
 
             rule op_asm() -> IrAstOperation
@@ -160,8 +182,9 @@ mod ir_builder {
                 }
 
             rule op_get_ptr() -> IrAstOperation
-                = "get_ptr" _ mut_ptr() ty:ast_ty() name:id() {
-                    IrAstOperation::GetPtr(name)
+                = "get_ptr" _ mut_ptr() ty:ast_ty() name:id()
+                    comma() ptr() ty:ast_ty() comma() offset:(decimal())  {
+                    IrAstOperation::GetPtr(name, ty, offset)
                 }
 
             rule op_insert_element() -> IrAstOperation
@@ -189,24 +212,29 @@ mod ir_builder {
                     IrAstOperation::Phi(pairs)
                 }
 
-            rule op_ptr_cast() -> IrAstOperation
-                = "ptr_cast" _ ptr() vn:id() "to" _ ptr() ty:ast_ty() {
-                    IrAstOperation::PtrCast(vn, ty)
-            }
-
             rule op_ret() -> IrAstOperation
                 = "ret" _ ty:ast_ty() vn:id() {
                     IrAstOperation::Ret(ty, vn)
                 }
 
-            rule op_state_load() -> IrAstOperation
-                = "state_load" _ ptr() dst:id() comma() "key" _ key:id() {
-                    IrAstOperation::StateLoad(dst, key)
+            rule op_state_load_quad_word() -> IrAstOperation
+                = "state_load_quad_word" _ ptr() dst:id() comma() "key" _ ptr() _ key:id() {
+                    IrAstOperation::StateLoadQuadWord(dst, key)
                 }
 
-            rule op_state_store() -> IrAstOperation
-                = "state_store" _ ptr() src:id() comma() "key" _ key:id() {
-                    IrAstOperation::StateStore(src, key)
+            rule op_state_load_word() -> IrAstOperation
+                = "state_load_word" _ "key" _ ptr() _ key:id() {
+                    IrAstOperation::StateLoadWord(key)
+                }
+
+            rule op_state_store_quad_word() -> IrAstOperation
+                = "state_store_quad_word" _ ptr() src:id() comma() "key" _ ptr() _ key:id() {
+                    IrAstOperation::StateStoreQuadWord(src, key)
+                }
+
+            rule op_state_store_word() -> IrAstOperation
+                = "state_store_word" _ src:id() comma() "key" _ ptr() _ key:id() {
+                    IrAstOperation::StateStoreWord(src, key)
                 }
 
             rule op_store() -> IrAstOperation
@@ -268,16 +296,7 @@ mod ir_builder {
                 / "true" _ { IrAstConstValue::Bool(true) }
                 / "false" _ { IrAstConstValue::Bool(false) }
                 / "0x" s:$(['0'..='9' | 'a'..='f' | 'A'..='F']*<64>) _ {
-                    let mut bytes: [u8; 32] = [0; 32];
-                    let mut cur_byte: u8 = 0;
-                    for (idx, ch) in s.chars().enumerate() {
-                        cur_byte = (cur_byte << 4) | ch.to_digit(16).unwrap() as u8;
-                        if idx % 2 == 1 {
-                            bytes[idx / 2] = cur_byte;
-                            cur_byte = 0;
-                        }
-                    }
-                    IrAstConstValue::B256(bytes)
+                    IrAstConstValue::B256(string_to_hex::<32>(s))
                 }
                 / n:decimal() { IrAstConstValue::Number(n) }
                 / string_const()
@@ -434,6 +453,7 @@ mod ir_builder {
         ret_type: IrAstTy,
         locals: Vec<(IrAstTy, String, bool, Option<IrAstOperation>)>,
         blocks: Vec<IrAstBlock>,
+        selector: Option<[u8; 4]>,
     }
 
     #[derive(Debug)]
@@ -463,16 +483,17 @@ mod ir_builder {
         Const(IrAstConst),
         ExtractElement(String, IrAstTy, String),
         ExtractValue(String, IrAstTy, Vec<u64>),
-        GetPtr(String),
+        GetPtr(String, IrAstTy, u64),
         InsertElement(String, IrAstTy, String, String),
         InsertValue(String, IrAstTy, String, Vec<u64>),
         Load(String),
         Nop,
         Phi(Vec<(String, String)>),
-        PtrCast(String, IrAstTy),
         Ret(IrAstTy, String),
-        StateLoad(String, String),
-        StateStore(String, String),
+        StateLoadQuadWord(String, String),
+        StateLoadWord(String),
+        StateStoreQuadWord(String, String),
+        StateStoreWord(String, String),
         Store(String, String),
     }
 
@@ -653,7 +674,7 @@ mod ir_builder {
             fn_decl.name,
             args.clone(),
             ret_type,
-            None,
+            fn_decl.selector,
             false,
         );
 
@@ -805,9 +826,15 @@ mod ir_builder {
                         opt_ins_md_idx,
                     )
                 }
-                IrAstOperation::GetPtr(src_name) => block
-                    .ins(context)
-                    .get_ptr(*ptr_map.get(&src_name).unwrap(), opt_ins_md_idx),
+                IrAstOperation::GetPtr(base_ptr, ptr_ty, offset) => {
+                    let ptr_ir_ty = ptr_ty.to_ir_type(context);
+                    block.ins(context).get_ptr(
+                        *ptr_map.get(&base_ptr).unwrap(),
+                        ptr_ir_ty,
+                        offset,
+                        opt_ins_md_idx,
+                    )
+                }
                 IrAstOperation::InsertElement(aval, ty, val, idx) => {
                     let ir_ty = ty.to_ir_aggregate_type(context);
                     block.ins(context).insert_element(
@@ -842,26 +869,30 @@ mod ir_builder {
                     }
                     block.get_phi(context)
                 }
-                IrAstOperation::PtrCast(value_name, ty) => {
-                    let ty = ty.to_ir_type(context);
-                    block.ins(context).ptr_cast(
-                        *val_map.get(&value_name).unwrap(),
-                        ty,
-                        opt_ins_md_idx,
-                    )
-                }
                 IrAstOperation::Ret(ty, ret_val_name) => {
                     let ty = ty.to_ir_type(context);
                     block
                         .ins(context)
                         .ret(*val_map.get(&ret_val_name).unwrap(), ty, opt_ins_md_idx)
                 }
-                IrAstOperation::StateLoad(dst, key) => block.ins(context).state_load(
-                    *val_map.get(&dst).unwrap(),
-                    *val_map.get(&key).unwrap(),
-                    opt_ins_md_idx,
-                ),
-                IrAstOperation::StateStore(src, key) => block.ins(context).state_store(
+                IrAstOperation::StateLoadQuadWord(dst, key) => {
+                    block.ins(context).state_load_quad_word(
+                        *val_map.get(&dst).unwrap(),
+                        *val_map.get(&key).unwrap(),
+                        opt_ins_md_idx,
+                    )
+                }
+                IrAstOperation::StateLoadWord(key) => block
+                    .ins(context)
+                    .state_load_word(*val_map.get(&key).unwrap(), opt_ins_md_idx),
+                IrAstOperation::StateStoreQuadWord(src, key) => {
+                    block.ins(context).state_store_quad_word(
+                        *val_map.get(&src).unwrap(),
+                        *val_map.get(&key).unwrap(),
+                        opt_ins_md_idx,
+                    )
+                }
+                IrAstOperation::StateStoreWord(src, key) => block.ins(context).state_store_word(
                     *val_map.get(&src).unwrap(),
                     *val_map.get(&key).unwrap(),
                     opt_ins_md_idx,
@@ -941,6 +972,19 @@ mod ir_builder {
         }
         Ok(())
     }
+
+    fn string_to_hex<const N: usize>(s: &str) -> [u8; N] {
+        let mut bytes: [u8; N] = [0; N];
+        let mut cur_byte: u8 = 0;
+        for (idx, ch) in s.chars().enumerate() {
+            cur_byte = (cur_byte << 4) | ch.to_digit(16).unwrap() as u8;
+            if idx % 2 == 1 {
+                bytes[idx / 2] = cur_byte;
+                cur_byte = 0;
+            }
+        }
+        bytes
+    }
 }
 
 // -------------------------------------------------------------------------------------------------
