@@ -375,15 +375,21 @@ fn instruction_to_doc<'a>(
                     Some(_) => Doc::text(md_namer.meta_as_string(context, span_md_idx, true)),
                 }),
             ),
-            Instruction::GetPointer(ptr) => {
+            Instruction::GetPointer {
+                base_ptr,
+                ptr_ty,
+                offset,
+            } => {
                 let name = block
                     .get_function(context)
-                    .lookup_local_name(context, ptr)
+                    .lookup_local_name(context, base_ptr)
                     .unwrap();
                 Doc::text_line(format!(
-                    "{} = get_ptr {}{}",
+                    "{} = get_ptr {}, ptr {}, {}{}",
                     namer.name(context, ins_value),
-                    ptr.as_string(context, name),
+                    base_ptr.as_string(context, name),
+                    ptr_ty.as_string(context),
+                    offset,
                     md_namer.meta_as_string(context, span_md_idx, true),
                 ))
             }
@@ -461,13 +467,6 @@ fn instruction_to_doc<'a>(
                     )
                 }
             }
-            Instruction::PointerCast(ptr_value, ty) => Doc::text_line(format!(
-                "{} = ptr_cast ptr {} to ptr {}{}",
-                namer.name(context, ins_value),
-                namer.name(context, ptr_value),
-                ty.as_string(context),
-                md_namer.meta_as_string(context, span_md_idx, true)
-            )),
             Instruction::Ret(v, t) => {
                 maybe_constant_to_doc(context, md_namer, namer, v).append(Doc::text_line(format!(
                     "ret {} {}{}",
@@ -476,14 +475,26 @@ fn instruction_to_doc<'a>(
                     md_namer.meta_as_string(context, span_md_idx, true),
                 )))
             }
-            Instruction::StateLoad { load_val, key } => Doc::text_line(format!(
-                "state_load ptr {}, key {}{}",
+            Instruction::StateLoadQuadWord { load_val, key } => Doc::text_line(format!(
+                "state_load_quad_word ptr {}, key ptr {}{}",
                 namer.name(context, load_val),
                 namer.name(context, key),
                 md_namer.meta_as_string(context, span_md_idx, true),
             )),
-            Instruction::StateStore { stored_val, key } => Doc::text_line(format!(
-                "state_store ptr {}, key {}{}",
+            Instruction::StateLoadWord(key) => Doc::text_line(format!(
+                "{} = state_load_word key ptr {}{}",
+                namer.name(context, ins_value),
+                namer.name(context, key),
+                md_namer.meta_as_string(context, span_md_idx, true),
+            )),
+            Instruction::StateStoreQuadWord { stored_val, key } => Doc::text_line(format!(
+                "state_store_quad_word ptr {}, key ptr {}{}",
+                namer.name(context, stored_val),
+                namer.name(context, key),
+                md_namer.meta_as_string(context, span_md_idx, true),
+            )),
+            Instruction::StateStoreWord { stored_val, key } => Doc::text_line(format!(
+                "state_store_word {}, key ptr {}{}",
                 namer.name(context, stored_val),
                 namer.name(context, key),
                 md_namer.meta_as_string(context, span_md_idx, true),
